% Computes 2-norm relative error
% bounds for mixed polynomial-rational approximation 
% of psi_1(A).
%
% Here A is a tridiagonal matrix as in Table 2
% in the manuscript.

f=@(z)(z./(exp(z)-1));

N=100; % upper limit on s

n=256;
ns=3;
a=full(gallery('tridiag',n,-1,4,-1));
[v,d]=eig(a);
b=v*diag(f(diag(d)))*v';
nb=norm(b);

Acoeff=norm(a/(2*pi))^(2*ns+2);

epsbound=zeros(1,N);

for s=1:N

sumzeta=0;
for k=1:s
    sumzeta=sumzeta+1/(k^(2*ns+2));
end

epsbound(s)=2*Acoeff*(zeta(2*ns+2)-sumzeta)/nb;
end

plot(1:N,log10(epsbound),'bd-')