function y=matxvec(p,q,a,g,h,b,d,x)
% Fast multiplication of a quasiseparable matrix times a vector: y = A x.
% Input: quasiseparable generators for matrix A, vector x. 
% Output: vector y.
%
% This function is for the scalar case. See matxvec_blocks if A is given 
% by a  block representation.
%
% This is an implementation of Algorithm 13.1 from Eidelman, Gohberg,
% Haimovici, "Separable Type Representation of Matrices and Fast
% Algorithms", vol. 1. See also section 1.3.
%
% Warning: no check on size consistency.

N=length(x);
yL=zeros(N,1);
yU=zeros(N,1);

yL(1)=0;
khi=q(:,1)*x(1);
yL(2)=p(2,:)*khi;
for i=3:N
    khi=a(:,:,i-1)*khi+q(:,i-1)*x(i-1);
    yL(i)=p(i,:)*khi;
end

yD=d.*x;

yU(N)=0;
eta=h(:,N)*x(N);
yU(N-1)=g(N-1,:)*eta;

for i=N-2:-1:1
    eta=b(:,:,i+1)*eta+h(:,i+1)*x(i+1);
    yU(i)=g(i,:)*eta;
end

y=yL+yD+yU;

end