function pmix=qs_mixed_approx(a, p, q, a_gen, g, h, b_gen, d, v, np, ns)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Mixed polynomial-rational approximation
% of psi_1(A)*v. Uses quasiseparable structure
% and structured solution of shifted qs systems.
% Last modification on Aug 23, 2020.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=length(d);   % matrix size


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute mixed approximation.
% Polynomial part:
if np>0
    xx=bernoulli(0:2*np); 
    pmixpoly=v;
    aa=a; 
    for j=2:2*np+1
        pmixpoly=pmixpoly+(xx(j)/factorial(j-1))*aa*v;
        aa=aa*a;
    end
else
    pmixpoly=v-a*v/2;
end

% Rational part:
ap2=(a/(2*pi))^2;
sigma=(1:ns).^2;
solutions=qs_shifted_function(p,q,a_gen,g,h,b_gen,d,sigma,v);
solutions=solutions*(1./(sigma.^np))';
prat=sum(solutions,2);
prat=(2*(-1)^np*ap2^(np+1))*prat;

% Mixed approximation:
pmix=pmixpoly+prat;
    
    



